@@ -35,6 +35,8 @@
 #define PREFIX_DRIVE "google:drive:"
 #define PREFIX_PICASAWEB "google:picasaweb:"
 
+static const guint MAX_RESULTS = 50;
+
 G_DEFINE_TYPE (GomGDataMiner, gom_gdata_miner, GOM_TYPE_MINER)
 
 static gchar *
@@ -550,30 +552,11 @@ account_miner_job_process_photo (GomAccountMinerJob *job,
 
   if (make != NULL || model != NULL)
     {
-      equipment_resource = gom_tracker_sparql_connection_ensure_resource
-        (job->connection,
-         job->cancellable, error,
-         &resource_exists,
-         job->datasource_urn, resource,
-         "nfo:Equipment", NULL);
-
-      if (*error != NULL)
-        goto out;
-
-      gom_tracker_sparql_connection_insert_or_replace_triple
-        (job->connection,
-         job->cancellable, error,
-         job->datasource_urn, equipment_resource,
-         "nfo:manufacturer", make);
-
-      if (*error != NULL)
-        goto out;
-
-      gom_tracker_sparql_connection_insert_or_replace_triple
-        (job->connection,
-         job->cancellable, error,
-         job->datasource_urn, equipment_resource,
-         "nfo:model", model);
+      equipment_resource = gom_tracker_utils_ensure_equipment_resource (job->connection,
+                                                                        job->cancellable,
+                                                                        error,
+                                                                        make,
+                                                                        model);
 
       if (*error != NULL)
         goto out;
@@ -809,34 +792,62 @@ query_gdata_documents (GomAccountMinerJob *job,
                        GDataDocumentsService *service,
                        GError **error)
 {
-  GDataDocumentsQuery *query;
-  GDataDocumentsFeed *feed;
+  GDataDocumentsQuery *query = NULL;
+  GDataDocumentsFeed *feed = NULL;
   GList *entries, *l;
+  gboolean succeeded_once = FALSE;
 
-  query = gdata_documents_query_new (NULL);
+  query = gdata_documents_query_new_with_limits (NULL, 1, MAX_RESULTS);
   gdata_documents_query_set_show_folders (query, TRUE);
-  feed = gdata_documents_service_query_documents
-    (service, query,
-     job->cancellable, NULL, NULL, error);
 
-  g_object_unref (query);
-
-  if (feed == NULL)
-    return;
-
-  entries = gdata_feed_get_entries (GDATA_FEED (feed));
-  for (l = entries; l != NULL; l = l->next)
+  while (TRUE)
     {
-      account_miner_job_process_entry (job, service, l->data, error);
+      GError *local_error;
 
-      if (*error != NULL)
+      local_error = NULL;
+      feed = gdata_documents_service_query_documents
+        (service, query,
+         job->cancellable, NULL, NULL, &local_error);
+      if (local_error != NULL)
         {
-          g_warning ("Unable to process entry %p: %s", l->data, (*error)->message);
-          g_clear_error (error);
+          if (succeeded_once)
+            {
+              g_warning ("Unable to query: %s", local_error->message);
+              g_error_free (local_error);
+            }
+          else
+            {
+              g_propagate_error (error, local_error);
+            }
+
+          break;
         }
+
+      succeeded_once = TRUE;
+
+      entries = gdata_feed_get_entries (GDATA_FEED (feed));
+      if (entries == NULL)
+        break;
+
+      for (l = entries; l != NULL; l = l->next)
+        {
+          local_error = NULL;
+          account_miner_job_process_entry (job, service, l->data, &local_error);
+
+          if (local_error != NULL)
+            {
+              g_warning ("Unable to process entry %p: %s", l->data, local_error->message);
+              g_error_free (local_error);
+            }
+        }
+
+      gdata_query_next_page (GDATA_QUERY (query));
+      g_clear_object (&feed);
     }
 
-  g_object_unref (feed);
+ out:
+  g_clear_object (&feed);
+  g_clear_object (&query);
 }
 
 static void
